/*-----------------------------------------------------------------------------
 Unit Name: sys_ini
 Author:    Alex Novikov
 Date:      16.02.04
 Purpose:      ,   ini .
 History:   
-----------------------------------------------------------------------------*/


//---------------------------------------------------------------------------
#ifndef sys_iniH
#define sys_iniH
//---------------------------------------------------------------------------

#include <system.hpp>
#include <sysutils.hpp>
#include <forms.hpp>
#include <vector>
using std::vector;

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

class Ini_file
{
public:

	Ini_file(AnsiString n = "")
	{
		filename_ = n;
		if(filename_=="") filename_ = get_app_name() + ".ini";
		if(ExtractFilePath(filename_)=="")
			filename_ = ExtractFilePath(Application->ExeName)+"\\"+filename_;
		get_sections();
//		if(filename_=="") filename_ = Application->ExeName + ".ini";
	}

	// - sections iterator -
	
	typedef vector<AnsiString>::iterator section_iterator; 
	vector<AnsiString>::iterator begin_section(){return sections_.begin();};
	vector<AnsiString>::iterator end_section(){return sections_.end();}
	
	// - keys iterator -
	
	typedef vector<AnsiString>::iterator key_iterator; 
	vector<AnsiString>::iterator begin_key(AnsiString s){get_keys(s);return keys_.begin();};
	vector<AnsiString>::iterator end_key(){return keys_.end();}
	
	// - get -
	
	AnsiString get(AnsiString group,AnsiString key,AnsiString def="")
	{
		const int bufsize = 200000;
		static char buf[bufsize];
		GetPrivateProfileString (group.c_str(), key.c_str(), def.c_str(), buf, bufsize, filename_.c_str());
		return buf;
	}

	bool get_bool(AnsiString group,AnsiString key,bool def=false)
	{
		return StrToBool(get(group,key,BoolToStr(def)));
	}

	int get_int(AnsiString group,AnsiString key,int def=-1)
	{
		return StrToInt(get(group,key,IntToStr(def)));
	}

	Extended get_float(AnsiString group,AnsiString key,Extended def=-1.0)
	{
		return StrToFloat(get(group,key,FloatToStr(def)));
	}

	TDateTime get_date(AnsiString group,AnsiString key)//,TDateTime def = Date())
	{
//		return StrToDateTime(get(group,key,DateTimeToStr(def)));
		return StrToDateTime(get(group,key,DateTimeToStr(Date())));
	}

	// - set -
	
	void set(AnsiString group,AnsiString key, AnsiString val)
	{
		bool b = WritePrivateProfileString(group.c_str(), key.c_str(), val.c_str(), filename_.c_str());
	}

	void set_bool(AnsiString group,AnsiString key, bool val)
	{
		set(group,key,BoolToStr(val));
	}

	void set_int(AnsiString group,AnsiString key, int val)
	{
		set(group,key,IntToStr(val));
	}

	void set_float(AnsiString group,AnsiString key, Extended val)
	{
		set(group,key,FloatToStr(val));
	}

	void set_date(AnsiString group,AnsiString key, TDateTime val)
	{
		set(group,key,DateTimeToStr(val));
	}
	
private:

	AnsiString filename_;
	vector<AnsiString> sections_;
	vector<AnsiString> keys_;


	void get_keys(AnsiString sect)
	{
		const int bufsize = 4096;
		static char buf[bufsize];
		int cnt  = GetPrivateProfileSection(sect.c_str(),buf,bufsize,filename_.c_str());
		keys_.clear();
		if(cnt==0) return;
		AnsiString b="";
		for(int i=0;i!=cnt;i++)
		{
			if(buf[i]=='\0')
			{
				keys_.push_back(b.SubString(1,b.Pos("=")-1));
				b="";
				continue;
			}
			b+=buf[i];
		}
	}
		
	void get_sections()
	{
		const int bufsize = 4096;
		static char buf[bufsize];
		int cnt  = GetPrivateProfileSectionNames(buf,bufsize,filename_.c_str());
		sections_.clear();
		if(cnt==0) return;
		AnsiString b="";
		for(int i=0;i!=cnt;i++)
		{
			if(buf[i]=='\0')
			{
				sections_.push_back(b);
				b="";
				continue;
			}
			b+=buf[i];
		}
	}
	
	AnsiString get_app_name()
	{
		AnsiString n = Application->ExeName;
		AnsiString fnext = ExtractFileName(n);
		AnsiString fn = fnext.SubString(1, fnext.Length() - ExtractFileExt(n).Length());
		return fn;
	}

	
};

//---------------------------------------------------------------------------
#endif
